/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.user.common;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import de.ozgcloud.user.keycloak.KeycloakApiProperties;
import jakarta.enterprise.context.ApplicationScoped;
import jakarta.inject.Inject;
import jakarta.json.JsonArray;
import jakarta.json.JsonString;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.microprofile.jwt.JsonWebToken;

@ApplicationScoped
public class JwtUtil {

	public static final String ORGANSIATIONSEINHEIT_IDS_CLAIM = "organisationseinheitIds";
	public static final String USER_ID_CLAIM = "userId";
	public static final String EMAIL_CLAIM = "email";
	public static final String RESOURCE_ACCESS_CLAIM = "resource_access";

	static final String MAP_ROLES_KEY = "roles";

	@Inject
	JsonWebToken jwt;

	@Inject
	KeycloakApiProperties keycloakApiProperties;

	public Optional<String> getUserName() {
		return Optional.ofNullable(jwt.getSubject());
	}

	@SuppressWarnings("unchecked")
	public List<String> getOrganisationsEinheitIds() {
		return Optional.ofNullable(jwt.getClaim(ORGANSIATIONSEINHEIT_IDS_CLAIM))
				.map(List.class::cast)
				.orElseGet(Collections::emptyList);
	}

	public Optional<String> getUserId() {
		return Optional.ofNullable(jwt.getClaim(USER_ID_CLAIM))
				.map(String::valueOf);
	}

	public String getEmail() {
		return Optional.ofNullable(jwt.getClaim(EMAIL_CLAIM))
				.map(String::valueOf)
				.filter(StringUtils::isNotEmpty)
				.orElseGet(() -> null);
	}

	public List<String> getUserRoles() {
		return Optional.ofNullable(jwt.getClaim(RESOURCE_ACCESS_CLAIM))
				.map(Map.class::cast)
				.map(resourceMap -> resourceMap.get(keycloakApiProperties.client()))
				.map(Map.class::cast)
				.map(clientMap -> clientMap.get(MAP_ROLES_KEY))
				.map(JsonArray.class::cast)
				.map(this::getAsStringArray)
				.orElseGet(Collections::<String>emptyList);
	}

	private List<String> getAsStringArray(JsonArray rolesArray) {
		return rolesArray.stream()
				.map(JsonString.class::cast)
				.map(JsonString::getString)
				.toList();
	}
}