/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.user.common.errorhandling;

import java.util.UUID;

import de.ozgcloud.common.errorhandling.ExceptionUtil;
import jakarta.enterprise.context.ApplicationScoped;
import jakarta.ws.rs.NotFoundException;
import lombok.extern.log4j.Log4j2;

@Log4j2
@ApplicationScoped
public class ApiErrorUtil {

	public static final String RESOURCE_NOT_FOUND_MESSAGE_CODE = "resource.not_found";
	public static final String RUNTIME_MESSAGE_CODE = "generale.server_error";
	public static final String ACCESS_DENIED_MESSAGE_CODE = "generale.access_denied";
	public static final String SERVICE_UNAVAILABLE_MESSAGE_CODE = "generale.service_unavailable";

	public ApiError buildFromRuntimeException(RuntimeException e) {
		var exceptionId = createExceptionId();
		var messageWithExceptionId = ExceptionUtil.formatMessageWithExceptionId(e.getMessage(), exceptionId);
		LOG.error("RuntimeException on Request", messageWithExceptionId);

		return buildRuntimeApiError(messageWithExceptionId, exceptionId);
	}

	String createExceptionId() {
		return UUID.randomUUID().toString();
	}

	private ApiError buildRuntimeApiError(String message, String exceptionId) {
		return ApiError.builder().issue(createIssueForRuntimeException(message, exceptionId)).build();
	}

	private Issue createIssueForRuntimeException(String message, String exceptionId) {
		return Issue.builder().messageCode(RUNTIME_MESSAGE_CODE).message(message).exceptionId(exceptionId).build();
	}

	public ApiError buildFromNotFoundException(NotFoundException e) {
		return ApiError.builder().issue(buildIssueForNotFoundException(e)).build();
	}

	private Issue buildIssueForNotFoundException(NotFoundException e) {
		return Issue.builder().message(e.getMessage()).messageCode(RESOURCE_NOT_FOUND_MESSAGE_CODE).exceptionId(createExceptionId()).build();
	}

	public ApiError buildFromResourceNotFoundException(ResourceNotFoundException e) {
		return ApiError.builder().issue(buildIssueForResourceNotFoundException(e)).build();
	}

	private Issue buildIssueForResourceNotFoundException(ResourceNotFoundException e) {
		return Issue.builder().message(e.getMessage()).messageCode(RESOURCE_NOT_FOUND_MESSAGE_CODE).exceptionId(e.getExceptionId()).build();
	}

	public ApiError buildFromAccessForbiddenException(AccessForbiddenException e) {
		var exceptionId = createExceptionId();
		var messageWithExceptionId = ExceptionUtil.formatMessageWithExceptionId(e.getMessage(), exceptionId);
		LOG.error("Access Denied Exception", messageWithExceptionId);

		return ApiError.builder().issue(buildIssueForAccessDeniedException(messageWithExceptionId, exceptionId)).build();
	}

	private Issue buildIssueForAccessDeniedException(String message, String exceptionId) {
		return Issue.builder().messageCode(ACCESS_DENIED_MESSAGE_CODE).message(message).exceptionId(exceptionId).build();
	}

	public ApiError buildFromKeycloakUnavailableException(KeycloakUnavailableException e) {
		return ApiError.builder().issue(buildIssueForServiceUnavailable(e)).build();
	}

	private Issue buildIssueForServiceUnavailable(TechnicalException e) {
		return Issue.builder().message(e.getMessage()).messageCode(SERVICE_UNAVAILABLE_MESSAGE_CODE).exceptionId(e.getExceptionId()).build();
	}
}