/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.user;

import static de.ozgcloud.user.UserTestFactory.*;
import static io.restassured.RestAssured.*;
import static org.hamcrest.CoreMatchers.*;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;

import java.util.stream.Stream;

import org.jboss.resteasy.reactive.common.util.RestMediaType;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import de.ozgcloud.user.common.errorhandling.ApiErrorUtil;
import de.ozgcloud.user.common.errorhandling.ResourceNotFoundException;
import de.ozgcloud.user.settings.UserSettingsResource;
import io.quarkus.test.InjectMock;
import io.quarkus.test.junit.QuarkusTest;
import io.quarkus.test.junit.TestProfile;

@QuarkusTest
@TestProfile(UserProfileResourceTestProfile.class)
class UserProfileResourceITCase {

	private static final String HTTP_LOCALHOST = "http://localhost:9092";
	private static final String SEARCH_PATH = UserProfileResource.USERS_PATH + "/?" + UserProfileResource.PARAM_SEARCH_BY + "=";

	@InjectMock
	UserService userService;

	private String settingsUrlWithUserId = HTTP_LOCALHOST
			+ UserSettingsResource.SETTINGS_LINK_PATTERN.replace("{id}", UserTestFactory.ID_STR);
	private String urlWithUserId = HTTP_LOCALHOST
			+ UserProfileResource.USER_PROFILE_RESOURCE_PATH_TEMPLATE.replace("{id}", UserTestFactory.ID_STR);

	@DisplayName("Test get user by id")
	@Nested
	class TestGetUser {

		@BeforeEach
		void init() {
			when(userService.getById(any())).thenReturn(UserTestFactory.create());
		}

		@Test
		void shouldHaveSelfLink() {
			given()
					.when()
					.accept(RestMediaType.APPLICATION_HAL_JSON)
					.get(UserProfileResource.USERS_PATH + "/{id}", UserTestFactory.ID_STR)
					.then().statusCode(200)
					.body("_links.self.href", equalTo(urlWithUserId));
		}

		@Test
		void shouldHaveSettingsLink() {
			given()
					.when()
					.accept(RestMediaType.APPLICATION_HAL_JSON)
					.get(UserProfileResource.USERS_PATH + "/{id}", UserTestFactory.ID_STR)
					.then().statusCode(200)
					.body("_links.settings.href", equalTo(settingsUrlWithUserId));
		}

		@Test
		void shouldReturnNotFound() {
			var exception = new ResourceNotFoundException(User.class, "not_found");
			when(userService.getById(any())).thenThrow(exception);

			var response = given().when()
					.accept(RestMediaType.APPLICATION_HAL_JSON)
					.get(UserProfileResource.USERS_PATH + "/not_found");

			response.then()
					.statusCode(404)
					.body("issues[0].messageCode", equalTo(ApiErrorUtil.RESOURCE_NOT_FOUND_MESSAGE_CODE))
					.body("issues[0].message",
							equalTo("Functional error: Resource 'User' with id 'not_found' not found. (ExceptionId: " + exception.getExceptionId()
									+ ")"))
					.body("issues[0].exceptionId", equalTo(exception.getExceptionId()));
		}
	}

	@Nested
	class TestFindUsers {

		private static final String SEARCH_BY = "no_one";

		@BeforeEach
		void init() {
			when(userService.findUsers(any())).thenReturn(Stream.of(UserTestFactory.create()));
		}

		@Test
		void shouldGetEmptyList() {
			when(userService.findUsers(any())).thenReturn(Stream.empty());

			given()
					.when()
					.accept(RestMediaType.APPLICATION_HAL_JSON)
					.get(SEARCH_PATH + SEARCH_BY)
					.then()
					.statusCode(200)
					.body(is("{\"_embedded\":{\"userProfileList\":[]},\"_links\":{\"self\":{\"href\":\"http://localhost:9092/api/userProfiles\"}}}"));
		}

		@Test
		void shouldGetUserProfiles() {
			given()
					.when()
					.accept(RestMediaType.APPLICATION_HAL_JSON)
					.get(SEARCH_PATH + USER_NAME)
					.then()
					.statusCode(200);
		}

		@Test
		void shouldHaveSelfLink() {
			given()
					.when()
					.accept(RestMediaType.APPLICATION_HAL_JSON)
					.get(SEARCH_PATH + USER_NAME)
					.then()
					.statusCode(200)
					.body("_links.self.href", equalTo(HTTP_LOCALHOST + UserProfileResource.USERS_PATH));
		}

		@Test
		void shouldGetOneDeletedUser() {
			var user = UserTestFactory.createBuilder().deleted(true).build();
			when(userService.findInactiveUsers(any())).thenReturn(Stream.of(user));

			given()
					.when()
					.accept(RestMediaType.APPLICATION_HAL_JSON)
					.get(SEARCH_PATH + "Max" + "&deleted=true")
					.then()
					.statusCode(200)
					.body("_embedded.userProfileList.size()", is(1))
					.body("_embedded.userProfileList[0].deleted", is(true));
		}

		@Test
		void shouldGetOneNotDeletedUser() {
			var user = UserTestFactory.createBuilder().deleted(false).build();
			when(userService.findActiveUsers(any())).thenReturn(Stream.of(user));

			given()
					.when()
					.accept(RestMediaType.APPLICATION_HAL_JSON)
					.get(SEARCH_PATH + "Max" + "&deleted=false")
					.then()
					.statusCode(200)
					.body("_embedded.userProfileList.size()", is(1))
					.body("_embedded.userProfileList[0].deleted", is(false));
		}

		@Test
		void shouldGetDeletedAndNotDeletedUsers() {
			var user1 = UserTestFactory.createBuilder().deleted(false).build();
			var user2 = UserTestFactory.createBuilder().deleted(true).build();
			when(userService.findUsers(any())).thenReturn(Stream.of(user1, user2));

			given()
					.when()
					.accept(RestMediaType.APPLICATION_HAL_JSON)
					.get(SEARCH_PATH + "Max")
					.then()
					.statusCode(200)
					.body("_embedded.userProfileList.size()", is(2))
					.body("_embedded.userProfileList[0].deleted", is(false))
					.body("_embedded.userProfileList[1].deleted", is(true));
		}

		@DisplayName("verify list content")
		@Nested
		class VerifyListItems {
			private static final String USER_PROFILES_PATH = "_embedded.userProfileList[0]._links";

			@BeforeEach
			void init() {
				when(userService.findUsers(any())).thenReturn(Stream.of(UserTestFactory.create()));
			}

			@Test
			void shouldHaveSelfLink() {
				given()
						.when()
						.accept(RestMediaType.APPLICATION_HAL_JSON)
						.get(SEARCH_PATH + USER_NAME)
						.then()
						.statusCode(200)
						.body(USER_PROFILES_PATH + ".self.href", equalTo(urlWithUserId));
			}

			@Test
			void shouldHaveSettingsLink() {
				given()
						.when()
						.accept(RestMediaType.APPLICATION_HAL_JSON)
						.get(SEARCH_PATH + USER_NAME)
						.then().statusCode(200)
						.body(USER_PROFILES_PATH + ".settings.href", equalTo(settingsUrlWithUserId));
			}
		}
	}
}
