/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.user;

import static de.ozgcloud.user.UserTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.time.Instant;
import java.util.Optional;
import java.util.stream.Stream;

import org.apache.commons.lang3.StringUtils;
import org.bson.types.ObjectId;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.user.common.errorhandling.ResourceNotFoundException;
import de.ozgcloud.user.common.errorhandling.TechnicalException;
import de.ozgcloud.user.keycloak.KeycloakUserRemoteService;

class UserServiceTest {

	@InjectMocks
	@Spy
	private UserService service;

	@Mock
	private KeycloakUserRemoteService keycloakRemoteService;

	@Mock
	private UserRepository repository;

	@Nested
	class TestGetById {

		private final static String ID = "xxx";

		@BeforeEach
		void mockRepository() {
			when(repository.findById(anyString())).thenReturn(Optional.of(UserTestFactory.create()));
		}

		@Test
		void shouldCallRepository() {
			service.getById(ID);

			verify(repository).findById(anyString());
		}

		@Test
		void shouldThrowExceptionIfNotExists() {
			when(repository.findById(anyString())).thenReturn(Optional.empty());

			assertThatThrownBy(() -> service.getById(ID))
					.isInstanceOf(ResourceNotFoundException.class)
					.extracting(e -> ((ResourceNotFoundException) e).getId()).isEqualTo(ID);
		}
	}

	@Nested
	class TestSave {

		private final User user = UserTestFactory.create();

		@BeforeEach
		void init() {
			when(repository.refresh(any())).thenReturn(UserTestFactory.create());
		}

		@Test
		void shouldCallUpdate() {
			when(repository.findByExternalId(any())).thenReturn(Optional.of(user));

			service.save(user.toBuilder().firstName("Other").build());

			verify(repository).updateUser(any(User.class));
		}

		@Test
		void shouldCallPersist() {
			service.save(user);

			verify(repository).persist(any(User.class));
		}

		@Test
		void shouldCallFindByExternalId() {
			service.save(user);

			verify(repository, atLeastOnce()).findByExternalId(any());
		}

		@Test
		void shouldCallFindByEmail() {
			service.save(UserTestFactory.createBuilder().externalId(null).build());

			verify(repository, atLeastOnce()).findByEmail(anyString());
		}

		@Test
		void shouldNotCallFindByEmailOnEmptyEMail() {
			service.save(UserTestFactory.createBuilder().id(new ObjectId()).externalId(null).email(StringUtils.EMPTY).build());

			verify(repository, never()).findByEmail(anyString());
		}
	}

	@Nested
	class TestMarkUnsyncedUserAsDeleted {

		@Test
		void shouldCallRepository() {
			var lastSyncTimestamp = Instant.now().toEpochMilli();

			service.markUnsyncedUsersAsDeleted(lastSyncTimestamp);

			verify(repository).updateUnsyncedUsers(lastSyncTimestamp);
		}
	}

	@Nested
	class TestFindUsers {

		private static final String SEARCH_QUERY = "aaa";
		private static final int LIMIT = 50;

		@Test
		void shouldFindUsers() {
			service.findUsers(FindUserProfilesQuery.builder().searchBy(SEARCH_QUERY).limit(LIMIT).build());

			verify(repository).findUsers(SEARCH_QUERY, LIMIT);
		}
	}

	@Nested
	class TestFindByExternalId {

		@Test
		void shouldSearchInUserRepository() {
			when(repository.findByKeycloakId(anyString())).thenReturn(Optional.of(UserTestFactory.create()));

			service.findByExternalId(EXTERNAL_ID);

			verify(repository).findByKeycloakId(EXTERNAL_ID);
			verify(keycloakRemoteService, never()).findUserById(anyString());
		}

		@Test
		void shouldCallKeycloakRemoteService() {
			var user = UserTestFactory.create();
			when(keycloakRemoteService.findUserById(anyString())).thenReturn(Optional.of(user));
			when(service.saveAndSync(user)).thenReturn(user);

			service.findByExternalId(EXTERNAL_ID);

			verify(keycloakRemoteService).findUserById(EXTERNAL_ID);
		}

		@Test
		void shouldCallSaveAndSync() {
			var user = UserTestFactory.create();
			when(keycloakRemoteService.findUserById(anyString())).thenReturn(Optional.of(user));
			doReturn(user).when(service).saveAndSync(user);

			service.findByExternalId(EXTERNAL_ID);

			verify(service).saveAndSync(user);
		}

		@Test
		void shouldThrowException() {
			assertThrows(ResourceNotFoundException.class, () -> service.findByExternalId(EXTERNAL_ID));
		}
	}

	@Nested
	class TestSaveAndSync {

		@Test
		void shouldSaveUser() {
			var user = UserTestFactory.create();

			service.saveAndSync(user);

			verify(service).save(user);
		}

		@Test
		void shouldSyncUser() {
			var user = UserTestFactory.create();

			var saved = service.saveAndSync(user);

			verify(keycloakRemoteService, timeout(1000)).updateOzgCloudUserId(saved);
		}
	}

	@Nested
	class TestFindActiveUsers {

		private static final String SEARCH_BY = "search by";
		private static final int LIMIT = 50;

		@Test
		void shouldFindActiveUsers() {
			var query = FindUserProfilesQuery.builder()
					.searchBy(SEARCH_BY)
					.limit(LIMIT)
					.build();
			service.findActiveUsers(query);

			verify(repository).findUsersByDeleted(SEARCH_BY, false, LIMIT);
		}

		@Test
		void shouldFindActiveUsersWithOrganisationsEinheitId() {
			var query = FindUserProfilesQuery.builder()
					.searchBy(SEARCH_BY)
					.organisationsEinheitId(ORGANISTATIONSEINHEITEN_ID)
					.limit(LIMIT)
					.build();
			service.findActiveUsers(query);

			verify(repository).findUsersByDeletedAndOrganisationsEinheitId(SEARCH_BY, false, ORGANISTATIONSEINHEITEN_ID, LIMIT);

		}
	}

	@Nested
	class TestFindInactiveUsers {

		private static final String SEARCH_BY = "search by";
		private static final int LIMIT = 50;

		@Test
		void shouldInactiveFindUsers() {
			var query = FindUserProfilesQuery.builder()
					.searchBy(SEARCH_BY)
					.limit(LIMIT)
					.build();
			service.findInactiveUsers(query);

			verify(repository).findUsersByDeleted(SEARCH_BY, true, LIMIT);
		}

		@Test
		void shouldFindInactiveUsersWithOrganisationsEinheitId() {
			var query = FindUserProfilesQuery.builder()
					.searchBy(SEARCH_BY)
					.organisationsEinheitId(ORGANISTATIONSEINHEITEN_ID)
					.limit(LIMIT)
					.build();
			service.findInactiveUsers(query);

			verify(repository).findUsersByDeletedAndOrganisationsEinheitId(SEARCH_BY, true, ORGANISTATIONSEINHEITEN_ID, LIMIT);

		}
	}

	@Nested
	class TestFindAllInactiveUserIds {

		private final Stream<String> deletedUserIds = Stream.of(ID.toHexString());

		@BeforeEach
		void init() {
			when(repository.findAllDeletedUserIds()).thenReturn(deletedUserIds);
		}

		@Test
		void shouldCallRepository() {
			service.findAllInactiveUserIds();

			verify(repository).findAllDeletedUserIds();
		}

		@Test
		void shouldReturnStream() {
			var result = service.findAllInactiveUserIds();

			assertThat(result).hasSize(1).first().isEqualTo(ID.toHexString());
		}
	}

	@Nested
	class TestDeleteInactive {

		private final User user = UserTestFactory.create();

		@BeforeEach
		void init() {
			doReturn(user).when(service).getById(ID_STR);
			doNothing().when(service).assertInactiveUser(user);
		}

		@Test
		void shouldGetUser() {
			callService();

			verify(service).getById(ID_STR);
		}

		@Test
		void shouldAssertInactiveUser() {
			callService();

			verify(service).assertInactiveUser(user);
		}

		@Test
		void shouldDeleteUser() {
			callService();

			verify(repository).deleteById(ID_STR);
		}

		private void callService() {
			service.deleteInactive(ID_STR);
		}
	}

	@Nested
	class TestAssertInactiveUser {

		@Test
		void shouldThrowException() {
			var user = createBuilder().deleted(false).build();

			assertThatThrownBy(() -> service.assertInactiveUser(user)).isInstanceOf(
					TechnicalException.class);
		}

		@Test
		void shouldNotThrowException() {
			var user = createBuilder().deleted(true);

			assertThatNoException().isThrownBy(() -> service.assertInactiveUser(user.build()));
		}
	}
}