/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.user.common;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.Map;

import de.ozgcloud.user.common.callcontext.CallContextMetadataTestFactory;
import jakarta.json.Json;
import jakarta.json.JsonArray;

import org.eclipse.microprofile.jwt.JsonWebToken;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import com.cronutils.utils.StringUtils;
import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.user.UserRole;
import de.ozgcloud.user.common.callcontext.CurrentCallContextUserService;
import de.ozgcloud.user.keycloak.KeycloakApiProperties;

class JwtUtilTest {

	@Spy
	@InjectMocks
	private JwtUtil jwtUtil;

	@Mock
	private JsonWebToken jwt;

	@Mock
	private CurrentCallContextUserService contextUserService;

	@Mock
	private KeycloakApiProperties keycloakApiProperties;

	@DisplayName("Get userName")
	@Nested
	class TestGetUserName {

		@Test
		void shouldGetClaim() {
			jwtUtil.getUserName();

			verify(jwt).getSubject();
		}

		@Test
		void shouldReturnFilledOptionalIfExist() {
			var userNameInClaim = LoremIpsum.getInstance().getName();
			when(jwt.getSubject()).thenReturn(userNameInClaim);

			var userName = jwtUtil.getUserName();

			assertThat(userName).contains(userNameInClaim);
		}

		@Test
		void shouldReturnEmptyOptionalIfNotExists() {
			when(jwt.getSubject()).thenReturn(null);

			var userName = jwtUtil.getUserName();

			assertThat(userName).isEmpty();
		}
	}

	@DisplayName("Get organisationseinheit ids")
	@Nested
	class TestGetOrganisationsEinheitIds {

		@BeforeEach
		void init() {
			when(jwt.getClaim(anyString())).thenReturn(List.of(CallContextMetadataTestFactory.ORGANISATORISCHE_EINHEITEN_ID));
		}

		@Test
		void shouldGetClaim() {
			jwtUtil.getOrganisationsEinheitIds();

			verify(jwt).getClaim(JwtUtil.ORGANSIATIONSEINHEIT_IDS_CLAIM);
		}

		@Test
		void shouldReturnFilledListIfExist() {
			var orgaIdInClaim = "0815";
			when(jwt.getClaim(anyString())).thenReturn(List.of(orgaIdInClaim));

			var orgaIds = jwtUtil.getOrganisationsEinheitIds();

			assertThat(orgaIds).hasSize(1);
			assertThat(orgaIds.get(0)).isEqualTo(orgaIdInClaim);
		}

		@Test
		void shouldReturnEmptyListIfNotExists() {
			when(jwt.getClaim(anyString())).thenReturn(null);

			var orgaIds = jwtUtil.getOrganisationsEinheitIds();

			assertThat(orgaIds).isEmpty();
		}
	}

	@DisplayName("Get userId")
	@Nested
	class TestGetUserId {

		@Test
		void shouldGetClaim() {
			jwtUtil.getUserId();

			verify(jwt).getClaim(JwtUtil.USER_ID_CLAIM);
		}

		@Test
		void shouldReturnFilledOptionalIfExist() {
			var userIdInClaim = LoremIpsum.getInstance().getEmail();
			when(jwt.getClaim(anyString())).thenReturn(userIdInClaim);

			var userId = jwtUtil.getUserId();

			assertThat(userId).contains(userIdInClaim);
		}

		@Test
		void shouldReturnEmptyOptionalIfNotExists() {
			when(jwt.getClaim(anyString())).thenReturn(null);

			var userId = jwtUtil.getUserId();

			assertThat(userId).isEmpty();
		}
	}

	@DisplayName("Get email")
	@Nested
	class TestGetEmail {

		@Test
		void shouldGetClaim() {
			jwtUtil.getEmail();

			verify(jwt).getClaim(JwtUtil.EMAIL_CLAIM);
		}

		@Test
		void shouldSetIfExist() {
			var emailInClaim = LoremIpsum.getInstance().getEmail();
			when(jwt.getClaim(anyString())).thenReturn(emailInClaim);

			var email = jwtUtil.getEmail();

			assertThat(email).isEqualTo(emailInClaim);
		}

		@Test
		void shouldSetNullIfNotExists() {
			when(jwt.getClaim(anyString())).thenReturn(StringUtils.EMPTY);

			var email = jwtUtil.getEmail();

			assertThat(email).isNull();
		}
	}

	@DisplayName("Get user roles")
	@Nested
	class TestGetUserRoles {

		private static final String CLIENT = LoremIpsum.getInstance().getName();

		@BeforeEach
		void mockKeycloakClient() {
			when(keycloakApiProperties.client()).thenReturn(CLIENT);
		}

		@Nested
		class TestExistsInClaim {

			@BeforeEach
			void init() {
				when(jwt.getClaim(anyString())).thenReturn(Map.of(CLIENT, Map.of(JwtUtil.MAP_ROLES_KEY, JsonArray.EMPTY_JSON_ARRAY)));
			}

			@Test
			void shouldGetClaim() {
				jwtUtil.getUserRoles();

				verify(jwt).getClaim(JwtUtil.RESOURCE_ACCESS_CLAIM);
			}

			@Test
			void shouldGetClientFromKeycloakApiProperties() {
				jwtUtil.getUserRoles();

				verify(keycloakApiProperties).client();
			}

			@Test
			void shouldReturnEmptyList() {
				var roles = jwtUtil.getUserRoles();

				assertThat(roles).isEmpty();
			}
		}

		@Nested
		class TestNotExistsInClaim {

			private static final String USER_ROLE = UserRole.VERWALTUNG_USER;

			@BeforeEach
			void init() {
				var array = Json.createArrayBuilder()
						.add(Json.createValue(USER_ROLE))
						.build();

				when(jwt.getClaim(anyString())).thenReturn(Map.of(CLIENT, Map.of(JwtUtil.MAP_ROLES_KEY, array)));
			}

			@Test
			void shouldGetClaim() {
				jwtUtil.getUserRoles();

				verify(jwt).getClaim(JwtUtil.RESOURCE_ACCESS_CLAIM);
			}

			@Test
			void shouldGetClientFromKeycloakApiProperties() {
				jwtUtil.getUserRoles();

				verify(keycloakApiProperties).client();
			}

			@Test
			void shouldReturnRoles() {
				var roles = jwtUtil.getUserRoles();

				assertThat(roles).hasSize(1);
				assertThat(roles.get(0)).isEqualTo(USER_ROLE);
			}
		}
	}
}