/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.user.common.callcontext;

import static de.ozgcloud.user.common.callcontext.GrpcCallContextInterceptor.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.user.User;
import de.ozgcloud.user.UserTestFactory;
import de.ozgcloud.user.common.callcontext.GrpcCallContextInterceptor.*;
import de.ozgcloud.user.common.errorhandling.ResourceNotFoundException;
import io.grpc.Metadata;
import io.grpc.ServerCall;
import io.grpc.Status;

class GrpcCallContextInterceptorTest {

	@InjectMocks
	private GrpcCallContextInterceptor interceptor;
	@Mock
	private ServerCall.Listener<Object> delegate;
	@Mock
	private ServerCall<Object, Object> originCall;
	@Spy
	private CurrentCallContextUserService userService;

	@DisplayName("Log context setting listener")
	@Nested
	class TestLogContextSettingListenerTest {

		private LogContextSettingListener<?, ?, ?> listener;
		private Metadata headers = CallContextMetadataTestFactory.createMetadata();

		@BeforeEach
		void createListener() {
			listener = spy(interceptor.new LogContextSettingListener<>(delegate, originCall, headers));
		}

		@DisplayName("get request id")
		@Nested
		class TestGetRequestId {

			@Test
			void shouldReturnIdFromHeader() {
				var reqId = listener.getRequestId();

				assertThat(reqId).isEqualTo(CallContextMetadataTestFactory.REQUEST_ID);
			}

			@Test
			void shouldReturnNewRequestIdIfNoGiven() {
				headers.removeAll(GrpcUtil.createKeyOf(KEY_REQUEST_ID));

				var reqId = listener.getRequestId();

				assertThat(reqId).isNotBlank().isNotEqualTo(CallContextMetadataTestFactory.REQUEST_ID);
			}

		}

		@DisplayName("create call context user")
		@Nested
		class TestCreateCallContextUser {
			@Test
			void shouldFilledUser() {
				var user = buildListener().createCallContextUser();

				assertThat(user).usingRecursiveComparison().ignoringFields("email").isEqualTo(CallContextUserTestFactory.create());
			}

			@Test
			void shouldFillOrgaIdCollection() {
				Metadata metadata = CallContextMetadataTestFactory.createMetadata();
				metadata.put(GrpcUtil.createKeyOf(KEY_ACCESS_LIMITED_ORGAID), "orgaid_2".getBytes());

				var user = buildListener(metadata).createCallContextUser();

				assertThat(user.getOrganisatorischeEinheitenIds()).hasSize(2).contains(
						CallContextMetadataTestFactory.ORGANISATORISCHE_EINHEITEN_ID,
						"orgaid_2");
			}

			@Test
			void shouldFillEmptyListIfNoOrgaIds() {
				Metadata metadata = CallContextMetadataTestFactory.createMetadata();
				metadata.removeAll(GrpcUtil.createKeyOf(KEY_ACCESS_LIMITED_ORGAID));

				var user = buildListener(metadata).createCallContextUser();

				assertThat(user.getOrganisatorischeEinheitenIds()).isEmpty();
			}

			@DisplayName("for key isOrganisationEinheitenIdCheckNecessary")
			@Nested
			class TestIsOrganisationEinheitenIdCheckNecessary {

				@Test
				void shouldMapValueIfKeyIsPresent() {
					var metadata = CallContextMetadataTestFactory.createMetadata();

					var user = buildListener(metadata).createCallContextUser();

					assertThat(user.isOrganisationEinheitenIdCheckNecessary()).isTrue();
				}

				@Test
				void shouldMapFalseIfKeyIsNotPresent() {
					var metadata = CallContextMetadataTestFactory.createMetadata();
					metadata.removeAll(GrpcUtil.createKeyOf(KEY_ACCESS_LIMITED));

					var user = buildListener(metadata).createCallContextUser();

					assertThat(user.isOrganisationEinheitenIdCheckNecessary()).isFalse();
				}
			}

		}

		@DisplayName("start security context")
		@Nested
		class TestStartSecurityContext {

			@Test
			void shouldCreateCallContextUser() {
				listener.startSecurityContext();

				verify(listener).createCallContextUser();
			}

			@Test
			void shouldCallUserService() {
				listener.startSecurityContext();

				verify(userService).setCallContextUser(any());
			}
		}

		@DisplayName("clear security context")
		@Nested
		class TestClearSecurityContext {

			@Test
			void shouldCallUserService() {
				listener.clearSecurityContext();

				verify(userService).clearCallContextUser();
			}
		}

		@DisplayName("do surround on")
		@Nested
		class TestDoSurroundOn {

			private Runnable mockRunnable = mock(Runnable.class);

			@Test
			void shouldRunRunnable() {
				listener.doSurroundOn(mockRunnable);

				verify(mockRunnable).run();
			}

			@Test
			void shouldStartSecurityContext() {
				listener.doSurroundOn(mockRunnable);

				verify(listener).startSecurityContext();
			}

			@Test
			void shouldClearSecurityContext() {
				listener.doSurroundOn(mockRunnable);

				verify(listener).clearSecurityContext();
			}

			@DisplayName("catch exception")
			@Nested
			class TestException {

				@Test
				void shouldClearSecurityContextOnException() {
					listener.doSurroundOn(() -> {
						throw new RuntimeException();
					});

					verify(listener).clearSecurityContext();
				}

				@Test
				void shouldCloseCallWithStatusNotFound() {
					listener.doSurroundOn(() -> {
						throw new ResourceNotFoundException(User.class, UserTestFactory.ID);
					});

					verify(originCall, never()).close(eq(Status.UNKNOWN), any());
					verify(originCall).close(Status.NOT_FOUND, headers);
				}

				@Test
				void shouldCloseCallWithStatusUnknown() {
					listener.doSurroundOn(() -> {
						throw new RuntimeException();
					});

					verify(originCall, never()).close(eq(Status.NOT_FOUND), any());
					verify(originCall).close(Status.UNKNOWN, headers);
				}
			}
		}

		@Nested
		class TestOnFunctions {

			private LogContextSettingListener<Object, ?, ?> listener;

			@BeforeEach
			void initListener() {
				listener = buildListener();
			}

			@Test
			void onMessageShouldCallSurround() {
				listener.onMessage(new Object());

				verify(listener).doSurroundOn(any());
			}

			@Test
			void onHalfCloseShouldCallSurround() {
				listener.onHalfClose();

				verify(listener).doSurroundOn(any());
			}

			@Test
			void onCancelShouldCallSurround() {
				listener.onCancel();

				verify(listener).doSurroundOn(any());
			}

			@Test
			void onCompleteShouldCallSurround() {
				listener.onComplete();

				verify(listener).doSurroundOn(any());
			}

			@DisplayName("onReady")
			@Nested
			class TestOnReady {

				@DisplayName("with existing clientName")
				@Nested
				class TestWithExistingClientName {

					@Test
					void shouldCallDoSurroundOn() {
						listener.onReady();

						verify(listener).doSurroundOn(any());
					}
				}

				@DisplayName("with missing clientName")
				@Nested
				class TestWithMissingClientName {

					private LogContextSettingListener<?, ?, ?> listener;
					private Metadata headers = CallContextMetadataTestFactory.createMetadata();

					@BeforeEach
					void initListener() {
						headers.discardAll(GrpcUtil.createKeyOf(GrpcCallContextInterceptor.KEY_CLIENT_NAME));
						listener = spy(interceptor.new LogContextSettingListener<>(delegate, originCall, headers));
					}

					@Test
					void shouldHandleMissingClientName() {
						listener.onReady();

						verify(listener).handleMissingClientName();
						verify(originCall).close(any(), eq(headers));
					}

					@Test
					void shouldNotCallDoSurroundOn() {
						listener.onReady();

						verify(listener, never()).doSurroundOn(any());
					}
				}
			}
		}

		private LogContextSettingListener<Object, ?, ?> buildListener() {
			return buildListener(headers);
		}

		private LogContextSettingListener<Object, ?, ?> buildListener(Metadata headers) {
			return spy(interceptor.new LogContextSettingListener<>(delegate, originCall, headers));
		}
	}
}