/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.user.common.errorhandling;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import jakarta.ws.rs.NotFoundException;
import jakarta.ws.rs.core.UriInfo;

import org.apache.commons.lang3.StringUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.user.User;
import de.ozgcloud.user.UserTestFactory;

class ExceptionHandlerTest {

	@InjectMocks
	private ExceptionHandler handler = new ExceptionHandler();
	@Mock
	private ApiErrorUtil apiErrorUtil;
	@Mock
	private UriInfo uriInfo;

	@DisplayName("Handle runtime exception")
	@Nested
	class TestHandleRuntimeException {

		private final RuntimeException exception = new RuntimeException(LoremIpsum.getInstance().getWords(5));

		@BeforeEach
		void mockUtil() {
			when(apiErrorUtil.buildFromRuntimeException(any())).thenReturn(ApiError.builder().build());
		}

		@Test
		void shouldCallApiErrorUtilToBuildApiError() {
			handler.mapRuntimeException(exception);

			verify(apiErrorUtil).buildFromRuntimeException(exception);
		}

		@Test
		void shouldHaveStatus() {
			var response = handler.mapRuntimeException(exception);

			assertThat(response.getStatus()).isEqualTo(500);
		}
	}

	@DisplayName("Handle notfound exception")
	@Nested
	class TestHandleNotFoundException {
		private final String PATH = "/this_path_does_not_exist";

		private final NotFoundException exception = new NotFoundException(LoremIpsum.getInstance().getWords(5));

		@BeforeEach
		void mockUtil() {
			when(apiErrorUtil.buildFromNotFoundException(any())).thenReturn(ApiError.builder().build());
		}

		@BeforeEach
		void mockUriInfo() {
			when(uriInfo.getPath()).thenReturn(PATH);
		}

		@Test
		void shouldCallApiErrorUtilToBuildApiError() {
			handler.mapNotFoundException(exception);

			verify(apiErrorUtil).buildFromNotFoundException(exception);
		}

		@Test
		void shouldHaveStatus() {
			var response = handler.mapNotFoundException(exception);

			assertThat(response.getStatus()).isEqualTo(404);
		}
	}

	@DisplayName("Handle resource not found exception")
	@Nested
	class TestHandleResourceNotFoundException {

		private final ResourceNotFoundException exception = new ResourceNotFoundException(User.class, 42L);

		@BeforeEach
		void mockUtil() {
			when(apiErrorUtil.buildFromResourceNotFoundException(any())).thenReturn(ApiError.builder().build());
		}

		@Test
		void shouldCallApiErrorUtilToBuildApiError() {
			handler.mapResourceNotFoundException(exception);

			verify(apiErrorUtil).buildFromResourceNotFoundException(exception);
		}

		@Test
		void shouldHaveStatus() {
			var response = handler.mapResourceNotFoundException(exception);

			assertThat(response.getStatus()).isEqualTo(404);
		}
	}

	@DisplayName("Handle keycloak unavailable exception")
	@Nested
	class TestHandleKeycloakUnavailableException {

		private final KeycloakUnavailableException exception = new KeycloakUnavailableException(StringUtils.EMPTY, StringUtils.EMPTY,
				StringUtils.EMPTY,
				new Throwable());

		@BeforeEach
		void mockUtil() {
			when(apiErrorUtil.buildFromKeycloakUnavailableException(any())).thenReturn(ApiError.builder().build());
		}

		@Test
		void shouldCallApiErrorUtilToBuildApiError() {
			handler.mapKeycloakUnavailableException(exception);

			verify(apiErrorUtil).buildFromKeycloakUnavailableException(exception);
		}

		@Test
		void shouldHaveStatus() {
			var response = handler.mapKeycloakUnavailableException(exception);

			assertThat(response.getStatus()).isEqualTo(503);
		}
	}

	@DisplayName("Handle access forbidden exception")
	@Nested
	class TestHandleAccessForbiddenException {

		private final AccessForbiddenException exception = new AccessForbiddenException(User.class, UserTestFactory.ID);

		@BeforeEach
		void mockUtil() {
			when(apiErrorUtil.buildFromAccessForbiddenException(any())).thenReturn(ApiError.builder().build());
		}

		@Test
		void shouldCallApiErrorUtilToBuildApiError() {
			handler.mapAccessForbiddenException(exception);

			verify(apiErrorUtil).buildFromAccessForbiddenException(exception);
		}

		@Test
		void shouldHaveStatus() {
			var response = handler.mapAccessForbiddenException(exception);

			assertThat(response.getStatus()).isEqualTo(403);
		}
	}
}