/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.user.recipient;

import static de.ozgcloud.user.settings.NotificationsSendFor.*;
import static org.assertj.core.api.Assertions.*;

import de.ozgcloud.user.UserTestFactory;
import de.ozgcloud.user.settings.UserSettingsTestFactory;
import jakarta.inject.Inject;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.EnumSource;
import org.junit.jupiter.params.provider.EnumSource.Mode;

import de.ozgcloud.user.User;
import de.ozgcloud.user.common.MongoDbTestProfile;
import de.ozgcloud.user.settings.NotificationsSendFor;
import io.quarkus.test.junit.QuarkusTest;
import io.quarkus.test.junit.TestProfile;

@QuarkusTest
@TestProfile(MongoDbTestProfile.class)
class RecipientRepositoryITCase {

	@Inject
	RecipientRepository repository;

	@DisplayName("Find by organisationsEinheitId")
	@Nested
	class TestFindByOrganisationsEinheitId {

		@BeforeEach
		void clearDb() {
			repository.deleteAll();
		}

		@DisplayName("with given organisationsEinheitId")
		@Nested
		class TestWithGivenOrganisationsEinheitId {

			private static final User USER_NOTIFICATION_SEND_FOR_ALL = UserTestFactory.createBuilder().userSettings(
					UserSettingsTestFactory.createBuilder().notificationsSendFor(ALL).build())
					.build();

			@Test
			void shouldReturnOnMatch() {
				persist(USER_NOTIFICATION_SEND_FOR_ALL);

				var userList = repository.findByOrganisationsEinheitId(UserTestFactory.ORGANISTATIONSEINHEITEN_ID);

				assertThat(userList).hasSize(1);
				assertThat(userList.get(0)).usingRecursiveComparison().isEqualTo(USER_NOTIFICATION_SEND_FOR_ALL);
			}

			@Test
			void shouldReturnEmptyListOnNonMatch() {
				persist(USER_NOTIFICATION_SEND_FOR_ALL);

				var userList = repository.findByOrganisationsEinheitId("nonMatch");

				assertThat(userList).isEmpty();
			}
		}

		@DisplayName("by notificationSendFor")
		@Nested
		class TestByNotificationSendFor {

			private static final User USER_NOTIFICATION_SEND_FOR_ALL = UserTestFactory.createBuilder().userSettings(
					UserSettingsTestFactory.createBuilder().notificationsSendFor(ALL).build())
					.build();

			@Test
			void shouldReturnOnAll() {
				persist(USER_NOTIFICATION_SEND_FOR_ALL);

				var userList = repository.findByOrganisationsEinheitId(UserTestFactory.ORGANISTATIONSEINHEITEN_ID);

				assertThat(userList).hasSize(1);
				assertThat(userList.get(0)).usingRecursiveComparison().isEqualTo(USER_NOTIFICATION_SEND_FOR_ALL);
			}

			@ParameterizedTest
			@EnumSource(mode = Mode.EXCLUDE, names = { "ALL" })
			void shouldNotReturnOn(NotificationsSendFor sendFor) {
				var user = UserTestFactory.createBuilder().userSettings(
						UserSettingsTestFactory.createBuilder().notificationsSendFor(sendFor).build())
						.build();
				persist(user);

				var userList = repository.findByOrganisationsEinheitId(UserTestFactory.ORGANISTATIONSEINHEITEN_ID);

				assertThat(userList).isEmpty();
			}
		}

		private void persist(User user) {
			repository.persist(user);
		}
	}
}