/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.user.settings;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.UUID;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.user.UserService;
import de.ozgcloud.user.UserTestFactory;

class UserSettingsServiceTest {

	@InjectMocks
	UserSettingsService service;
	@Mock
	UserSettingsRepository repository;

	@Mock
	UserService userService;

	@DisplayName("Update by user id")
	@Nested
	class TestUpdateByUserId {

		private final UserSettings userSettings = UserSettingsTestFactory.create();
		private final String userId = UUID.randomUUID().toString();

		@Test
		void shouldCallRepository() {
			service.updateByUserId(userSettings, userId);

			verify(repository).updateByUserId(userId, userSettings);
		}
	}

	@DisplayName("Get by userId")
	@Nested
	class TestGetByUserId {

		private final String userId = UUID.randomUUID().toString();

		@DisplayName("with existing user settings")
		@Nested
		class TestWithExistingUserSettings {

			@BeforeEach
			void mockUserService() {
				when(userService.getById(anyString())).thenReturn(
						UserTestFactory.createBuilder().userSettings(
								UserSettings.builder().notificationsSendFor(NotificationsSendFor.ALL).build()).build());
			}

			@Test
			void shouldCallUserServiceFindById() {
				service.getByUserId(userId);

				verify(userService).getById(userId);
			}

			@Test
			void shouldReturnExistingUserSettingsIfExists() {
				var userSettings = service.getByUserId(userId);

				assertThat(userSettings.getNotificationsSendFor()).isEqualTo(NotificationsSendFor.ALL);
			}
		}

		@DisplayName("with non existing user settings")
		@Nested
		class TestWithNonExistingUserSettings {

			@BeforeEach
			void mockUserService() {
				when(userService.getById(anyString()))
						.thenReturn(UserTestFactory.createBuilder().userSettings(null).build());
			}

			@Test
			void shouldCallUserServiceFindById() {
				service.getByUserId(userId);

				verify(userService).getById(userId);
			}

			@Test
			void shouldReturnInitialUserSettingsIfNotExists() {
				var userSettings = service.getByUserId(userId);

				assertThat(userSettings.getNotificationsSendFor()).isEqualTo(NotificationsSendFor.NONE);
			}
		}
	}
}