/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.user.sync;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.time.Instant;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.user.common.lock.Lock;
import de.ozgcloud.user.common.lock.LockService;

class SyncSchedulerTest {

	@Spy
	@InjectMocks
	private SyncScheduler scheduler;
	@Mock
	private SyncService service;
	@Mock
	private LockService syncLockService;

	@DisplayName("Start Scheduler")
	@Nested
	class TestStart {

		@BeforeEach
		void init() {
			when(syncLockService.isNotLocked()).thenReturn(true);
			when(syncLockService.lockCollection()).thenReturn(Lock.builder().timestamp(Instant.now().toEpochMilli()).build());
		}

		@Test
		void shouldDoSyncIfNotLocked() {
			scheduler.start();

			verify(service).sync(anyLong());
		}

		@Test
		void shouldCallLockServiceLockCollection() {
			scheduler.start();

			verify(syncLockService).lockCollection();
		}

		@Test
		void shouldUnlockLock() {
			scheduler.start();

			verify(syncLockService).unlock();
		}

		@Test
		void shouldUnlockLockOnException() {
			doThrow(new RuntimeException()).when(service).sync(anyLong());

			try {
				scheduler.start();
			} catch (Exception e) {
				verify(syncLockService).unlock();
			}
		}
	}

	@DisplayName("If Locked")
	@Nested
	class TestIfLocked {

		@BeforeEach
		void mockTimeStamp() {
			when(syncLockService.isNotLocked()).thenReturn(false);
			when(syncLockService.isLockOlderThanADay()).thenReturn(true);

		}

		@Test
		void shouldNotDoSync() {
			scheduler.start();

			verify(service, never()).sync(anyLong());
		}

		@Test
		void shouldCheckIfLockIsOlderThanADay() {
			scheduler.start();

			verify(syncLockService).isLockOlderThanADay();
		}

		@Test
		void shouldRemoveLockWhenLockIsOlderThanADay() {
			scheduler.start();

			verify(syncLockService).unlock();
		}
	}
}